\ ==============================================================================
\
\           acn - the AVL binary tree cell node module in the ffl
\
\               Copyright (C) 2006  Dick van Oudheusden
\  
\ This library is free software; you can redistribute it and/or
\ modify it under the terms of the GNU General Public
\ License as published by the Free Software Foundation; either
\ version 2 of the License, or (at your option) any later version.
\
\ This library is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\ General Public License for more details.
\
\ You should have received a copy of the GNU General Public
\ License along with this library; if not, write to the Free
\ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\
\ ==============================================================================
\ 
\  $Date: 2006/09/17 19:03:05 $ $Revision: 1.1 $
\
\ ==============================================================================


include ffl/config.fs


[UNDEFINED] acn.version [IF]


include ffl/stc.fs
include ffl/bcn.fs


( acn = AVL binary tree cell node module )
( The acn module implements the node in an AVL binary tree. It extends the     )
( bcn structure. )


1 constant acn.version


( AVL Tree node structure )

struct: acn%       ( - n = Get the required space for the acn structure )
  bcn% field: acn>node            \ the general node
       cell:  acn>balance         \ the balance factor (-1,0,1)
;struct



( Structure creation, initialisation and destruction )

: acn-init         ( w:parent w:data w:key w:acn - = Initialise the acn structure with a key, data and parent )
  >r
  r@ bcn-init
  r> acn>balance 0!
;


: acn-new          ( w:data w:key - w:acn = Create a new AVL tree node on the heap )
  acn% allocate  throw   >r r@ acn-init r>
;


: acn-free         ( w:acn - = Free the AVL tree node from the heap )
  free throw 
;


( Private words )

: acn-balance@     ( w:acn - w:balance = Get the balance of the node )
  acn>balance @
;


( Inspection )

: acn-dump         ( w:acn - = Dump the AVL tree node structure )
  ." acn:" dup . cr
  dup acn>node bcn-dump
  ."   balance:" acn>balance ? cr
;

[THEN]

\ ==============================================================================
